package PlayBot::sites::youtube;

use strict;
use warnings;

use LWP::UserAgent;
use JSON;
use FindBin;

my $conf;
my $endpoint = "https://www.googleapis.com/youtube/v3";

BEGIN {
    chdir "$FindBin::Bin/";
    local $/;
    open CONF, '<', 'playbot.conf';
    my $json = <CONF>;
    $conf = decode_json($json);
}

sub regex_playlist
{
    return qr#(?:^|[^!])https?://www.youtube.com/playlist\?[a-zA-Z0-9_=&-]*list=([a-zA-Z0-9_-]+)#;
}

sub regex
{
    return qr#(?:^|[^!])https?://(?:www.youtube.com/watch\?[a-zA-Z0-9_=&-]*v=|youtu.be/)([a-zA-Z0-9_-]+)#;
}

sub get_playlist
{
    shift;
	my $id = shift;
	
    my $ua = LWP::UserAgent->new(
        timeout     => 30,
        env_proxy   => 1,
    );
	my $response = $ua->get($endpoint
        .'/playlists?id='.$id
        .'&key='.$conf->{'youtube_api_key'}
        .'&part=snippet,contentDetails');
	die($response->status_line) unless ($response->is_success);

	my $content = decode_json($response->decoded_content);

    die "playlist not found" if (not scalar @{ $content->{items} });
    
    my $playlist = $content->{'items'}->[0];
    my %infos;

    $infos{'title'} = $playlist->{'snippet'}->{'title'};
    $infos{'duration'} = $playlist->{'contentDetails'}->{'itemCount'};
    $infos{'author'} = $playlist->{'snippet'}->{'channelTitle'};
	$infos{'site'} = 'youtube';
    $infos{'url'} = 'https://www.youtube.com/playlist?list='.$id;

	$response = $ua->get($endpoint
        .'/playlistItems?playlistId='.$id
        .'&key='.$conf->{'youtube_api_key'}
        .'&part=snippet');
	die($response->status_line) unless ($response->is_success);

	$content = decode_json($response->decoded_content);

    die "empty playlist" if (not scalar @{ $content->{items} });

    my @urls;
    foreach my $subcontent (@{ $content->{items} })
    {
        my $subid = $subcontent->{snippet}{resourceId}{videoId};
        push @urls, 'https://www.youtube.com/watch?v='.$subid;
    }

    $infos{urls} = \@urls;
	return %infos;
}

sub get
{
    shift;
	my $id = shift;
	
    my $ua = LWP::UserAgent->new(
        timeout     => 30,
        env_proxy   => 1,
    );
	my $response = $ua->get($endpoint
        .'/videos?id='.$id
        .'&key='.$conf->{'youtube_api_key'}
        .'&part=snippet,contentDetails');
	die($response->status_line) unless ($response->is_success);

	my $content = decode_json($response->decoded_content);

    die "video not found" if (not scalar @{ $content->{items} });
    
    my $video = $content->{'items'}->[0];
    my %infos;

    $infos{'title'} = $video->{'snippet'}->{'title'};

    $video->{'contentDetails'}->{'duration'} =~ /PT((?<h>\d+)H)?((?<m>\d+)M)?((?<s>\d+)S)?/;

    # default values
    my ($h) = $+{'h'} || 0;
    my ($m) = $+{'m'} || 0;
    my ($s) = $+{'s'} || 0;

    # there is one more second, don't know why
    $infos{'duration'} = $h*3600 + $m*60 + $s - 1;

    $infos{'author'} = $video->{'snippet'}->{'channelTitle'};
	$infos{'site'} = 'youtube';
    $infos{'url'} = 'https://www.youtube.com/watch?v='.$id;

	return %infos;
}

1;
